@@ -14,15 +14,14 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Collector::RRDStorage;
+use strict;
+use warnings;
 
 use Torrus::ConfigTree;
 use Torrus::Log;
-
-use strict;
 use RRDs;
 
 our $useThreads;
@@ -87,6 +86,7 @@ sub initThreads
         $thrUpdateThread->detach();
         $threadsInUse = 1;
     }
+    return;
 }
 
 
@@ -110,6 +110,7 @@ sub initTarget
 
     $sref->{'byfile'}{$filename}{$token} = 1;
     $sref->{'filename'}{$token} = $filename;
+    return 1;
 }
 
 
@@ -129,6 +130,7 @@ sub setValue
     my $sref = $collector->storageData( 'rrd' );
 
     $sref->{'values'}{$token} = [$value, $timestamp, $uptime];
+    return;
 }
 
 
@@ -169,6 +171,7 @@ sub storeData
     }
 
     delete $sref->{'values'};
+    return;
 }
 
 
@@ -178,6 +181,7 @@ sub semaphoreDown
     {
         $rrdtoolSemaphore->down();
     }
+    return;
 }
 
 sub semaphoreUp
@@ -186,6 +190,7 @@ sub semaphoreUp
     {
         $rrdtoolSemaphore->up();
     }
+    return;
 }
 
 
@@ -320,6 +325,7 @@ sub createRRD
     Error("ERROR creating $filename: $err") if $err;
     
     delete $sref->{'rrdinfo_ds'}{$filename};
+    return;
 }
 
 
@@ -423,7 +429,7 @@ sub updateRRD
     &Torrus::DB::checkInterrupted();
 
     # Build the arguments for RRDs::update.
-    my $template;
+    my $template = '';
     my $values;
 
     # We will use the average timestamp
@@ -519,6 +525,7 @@ sub updateRRD
             delete $sref->{'rrdinfo_ds'}{$filename};
         }
     }
+    return;
 }
 
 
@@ -526,7 +533,6 @@ sub updateRRD
 sub rrdUpdateThread
 {
     &Torrus::DB::setSafeSignalHandlers();
-    $| = 1;
     &Torrus::Log::setTID( threads->tid() );
     
     my $cmdlist;
@@ -556,6 +562,7 @@ sub rrdUpdateThread
             $thrErrorsQueue->enqueue( $cmdlist->[0] );
         }
     }
+    return;
 }
 
 
@@ -579,6 +586,7 @@ sub deleteTarget
     }
 
     delete $sref->{'values'}{$token};
+    return;
 }
 
 
