@@ -16,14 +16,21 @@
 
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
+# this policy complains about accessing private methods in Net::SNMP
+## no critic (Variables::ProtectPrivateVars)
+
+# this policy complains about regexp result untested, although it is tested
+## no critic (RegularExpressions::ProhibitCaptureWithoutTest)
+
 package Torrus::Collector::SNMP;
+use strict;
+use warnings;
 
 use Torrus::Collector::SNMP_Params;
 use Torrus::ConfigTree;
 use Torrus::Log;
 use Torrus::SNMP_Failures;
 
-use strict;
 use Net::hostent;
 use Socket;
 use Net::SNMP qw(:snmp);
@@ -60,6 +67,7 @@ $Torrus::Collector::params{'snmp'} = {
     'snmp-object-type'  => 'OTHER',
     'snmp-check-sysuptime' => 'yes',
     'snmp-max-msg-size' => undef,
+    'snmp-maxrepetitions' => 10,
     'snmp-ignore-mib-errors' => undef,
     };
 
@@ -146,6 +154,7 @@ sub initCollectorGlobals
     {
         $mapsExpire{$maphash} = int( $now + rand( $mapsUpdateInterval ) );
     }    
+    return;
 }
 
 
@@ -243,17 +252,17 @@ sub initTargetAttributes
     $cref->{'oids_per_pdu'}{$hosthash} =
         $collector->param($token, 'snmp-oids-per-pdu');
 
-    if( $collector->param($token, 'snmp-object-type') eq 'COUNTER64' )
+    if( $collector->paramString($token, 'snmp-object-type') eq 'COUNTER64' )
     {
         $cref->{'64bit_oid'}{$oid} = 1;
     }
 
-    if( $collector->param($token, 'snmp-check-sysuptime') eq 'no' )
+    if( $collector->paramDisabled($token, 'snmp-check-sysuptime') )
     {
         $cref->{'nosysuptime'}{$hosthash} = 1;
     }
 
-    if( $collector->param($token, 'snmp-ignore-mib-errors') eq 'yes' )
+    if( $collector->paramEnabled($token, 'snmp-ignore-mib-errors') )
     {
         $cref->{'ignoremiberrors'}{$hosthash}{$oid} = 1;
     }
@@ -268,9 +277,9 @@ sub getHostHash
     my $token = shift;
 
     my $hostname = $collector->param($token, 'snmp-host');
-    my $domain = $collector->param($token, 'domain-name');
+    my $domain = $collector->paramString($token, 'domain-name');
     
-    if( length( $domain ) > 0 and
+    if( $domain ne '' and
         index($hostname, '.') < 0 and
         index($hostname, ':') < 0 )
     {
@@ -323,9 +332,10 @@ sub snmpSessionArgs
     
     foreach my $arg ( qw(-localaddr -localport) )
     {
-        if( defined( $collector->param($token, 'snmp' . $arg) ) )
+        my $val = $collector->param($token, 'snmp' . $arg);
+        if( defined( $val ) )
         {
-            push( @{$ret}, $arg, $collector->param($token, 'snmp' . $arg) );
+            push( @{$ret}, $arg, $val );
         }
     }
             
@@ -340,10 +350,10 @@ sub snmpSessionArgs
         foreach my $arg ( qw(-authkey -authpassword -authprotocol
                              -privkey -privpassword -privprotocol) )
         {
-            if( defined( $collector->param($token, 'snmp' . $arg) ) )
+            my $val = $collector->param($token, 'snmp' . $arg);            
+            if( defined( $val ) )
             {
-                push( @{$ret},
-                      $arg, $collector->param($token, 'snmp' . $arg) );
+                push( @{$ret}, $arg, $val );
             }
         }
     }
@@ -408,7 +418,7 @@ sub openNonblockingSession
         my $refcount = $Net::SNMP::Transport::SOCKETS->{
             $sock_name}->[&Net::SNMP::Transport::_SHARED_REFC()];
                                                                       
-        if( $refcount == 1 )
+        if( defined($refcount) and $refcount == 1 )
         {
             my $buflen = int($Torrus::Collector::SNMP::RxBuffer);
             my $socket = $session->transport()->socket();
@@ -599,7 +609,9 @@ sub lookupMap
 
         # Retrieve the map table
 
+        my $maxrepetitions = $collector->param($token, 'snmp-maxrepetitions');
         $session->get_table( -baseoid => $map,
+                             -maxrepetitions => $maxrepetitions,
                              -callback => [\&mapLookupCallback,
                                            $collector, $hosthash, $map] );
 
@@ -625,10 +637,10 @@ sub lookupMap
             if( defined ( $maps{$hosthash}{$map} ) )
             {
                 Error("Current map follows");
-                while( my($key, $val) = each
+                while( my($mkey, $mval) = each
                        %{$maps{$hosthash}{$map}} )
                 {
-                    Error("'$key' => '$val'");
+                    Error("'$mkey' => '$mval'");
                 }
             }
             return 'notfound';
@@ -681,14 +693,17 @@ sub mapLookupCallback
         $session->close();
         probablyDead( $collector, $hosthash );
         return undef;
-    }    
+    }
+    return;
 }
 
+
 sub activeMappingSessions
 {
     return scalar( @mappingSessions );
 }
-    
+
+
 # The target host is unreachable. We try to reach it few more times and
 # give it the final diagnose.
 
@@ -826,6 +841,7 @@ sub hostReachableAgain
                                       scalar( keys %hostUnreachableSeen));
         }
     }
+    return;
 }
 
 
@@ -842,14 +858,17 @@ sub deleteTarget
     my $hosthash = $tref->{'hosthash'};    
     my $oid = $tref->{'oid'};
 
-    delete $cref->{'targets'}{$hosthash}{$oid}{$token};
-    if( not %{$cref->{'targets'}{$hosthash}{$oid}} )
+    if( defined($oid) )
     {
-        delete $cref->{'targets'}{$hosthash}{$oid};
-
-        if( not %{$cref->{'targets'}{$hosthash}} )
+        delete $cref->{'targets'}{$hosthash}{$oid}{$token};
+        if( scalar(keys %{$cref->{'targets'}{$hosthash}{$oid}}) == 0 )
         {
-            delete $cref->{'targets'}{$hosthash};
+            delete $cref->{'targets'}{$hosthash}{$oid};
+            
+            if( scalar(keys %{$cref->{'targets'}{$hosthash}}) == 0 )
+            {
+                delete $cref->{'targets'}{$hosthash};
+            }
         }
     }
 
@@ -860,6 +879,7 @@ sub deleteTarget
         delete $cref->{'mapsDependentTokens'}{$maphash}{$token};
     }
     delete $cref->{'mapsRelatedMaps'}{$token};
+    return;
 }
 
 # Main collector cycle
@@ -1011,6 +1031,7 @@ sub runCollector
             %mapLookupScheduled = ();
         }
     }
+    return;
 }
 
 
@@ -1137,6 +1158,7 @@ sub callback
             }
         }
     }
+    return;
 }
 
 
@@ -1242,6 +1264,7 @@ sub postProcess
             $collector->deleteTarget($token);
         }
     }    
+    return;
 }
 
 
@@ -1314,6 +1337,7 @@ sub reachable_runCollector
             $collector->setValue( $token, $val, $timestamp, 0 );
         }
     }
+    return;
 }
 
         
