@@ -1,4 +1,4 @@
-#  Copyright (C) 2002-2010  Stanislav Sinyagin
+#  Copyright (C) 2002-2011  Stanislav Sinyagin
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,31 +14,38 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Core SNMP device discovery module
 
-package Torrus::DevDiscover::DevDetails;
-
 package Torrus::DevDiscover;
-
 use strict;
+use warnings;
+
 use POSIX qw(strftime);
 use Net::SNMP qw(:snmp :asn1);
 use Digest::MD5 qw(md5);
 
+use Torrus::DevDiscover::DevDetails;
 use Torrus::Log;
 
 BEGIN
 {
     foreach my $mod ( @Torrus::DevDiscover::loadModules )
     {
-        eval( 'require ' . $mod );
-        die( $@ ) if $@;
+        if( not eval('require ' . $mod) or $@ )
+        {
+            die($@);
+        }
     }
 }
 
+# Callback registry to inizialise multithreading before any threads are spawned
+our %threading_init_callbacks;
+
+# Callback registry to report discovery failures
+our %discovery_failed_callbacks;
+
 # Custom overlays for templates
 # overlayName ->
 #     'Module::templateName' -> { 'name' => 'templateName',
@@ -64,7 +71,8 @@ $defaultParams{'snmp-check-sysuptime'} = 'yes';
 $defaultParams{'show-recursive'} = 'yes';
 $defaultParams{'snmp-ipversion'} = '4';
 $defaultParams{'snmp-transport'} = 'udp';
-
+$defaultParams{'snmp-maxrepetitions'} = 10;
+    
 our @copyParams =
     ( 'collector-period',
       'collector-timeoffset',
@@ -77,6 +85,7 @@ our @copyParams =
       'monitor-period',
       'monitor-timeoffset',
       'nodeid-device',
+      'rrd-create-rra',
       'show-recursive',
       'snmp-host',
       'snmp-port',
@@ -98,6 +107,7 @@ our @copyParams =
       'snmp-oids-per-pdu',
       'snmp-check-sysuptime',
       'snmp-max-msg-size',
+      'snmp-maxrepetitions',
       'snmp-reachability-rra',
       'system-id' );
 
@@ -130,11 +140,15 @@ sub new
     foreach my $module ( 'Torrus::DevDiscover',
                          @Torrus::DevDiscover::loadModules )
     {
-        while( my($name, $oid) = each %{eval('\%'.$module.'::oiddef')} )
+        my $oiddef_ref = eval('\%'.$module.'::oiddef');
+        die($@) if $@;
+        if( ref($oiddef_ref) )
         {
-            die( $@ ) if $@;
-            $self->{'oiddef'}->{$name} = $oid;
-            $self->{'oidrev'}->{$oid} = $name;
+            while( my($name, $oid) = each %{$oiddef_ref} )
+            {
+                $self->{'oiddef'}->{$name} = $oid;
+                $self->{'oidrev'}->{$oid} = $name;
+            }
         }
     }
 
@@ -200,15 +214,12 @@ sub discover
             return 0;
         }
         $snmpargs{'-community'} = $community;
-
-        # set maxMsgSize to a maximum value for better compatibility
         
         my $maxmsgsize = $devdetails->param('snmp-max-msg-size');
         if( defined( $maxmsgsize ) )
         {
-            $devdetails->setParam('snmp-max-msg-size', $maxmsgsize);
             $snmpargs{'-maxmsgsize'} = $maxmsgsize;
-        }        
+        }
     }
     elsif( $version eq '3' )        
     {
@@ -286,11 +297,9 @@ sub discover
     $data->{'param'} = {};
 
     $data->{'templates'} = [];
-    my $customTmpl = $devdetails->param('custom-host-templates');
-    if( length( $customTmpl ) > 0 )
-    {
-        push( @{$data->{'templates'}}, split( /\s*,\s*/, $customTmpl ) );
-    }
+    push( @{$data->{'templates'}},
+          split( /\s*,\s*/,
+                 $devdetails->paramString('custom-host-templates') ) );
     
     # Build host-level legend
     my %legendValues =
@@ -317,14 +326,14 @@ sub discover
     foreach my $key ( sort keys %legendValues )
     {
         my $text = $legendValues{$key}{'value'};
-        if( length( $text ) > 0 )
+        if( defined($text) and $text ne '' )
         {
             $text = $devdetails->screenSpecialChars( $text );
             $legend .= $legendValues{$key}{'name'} . ':' . $text . ';';
         }
     }
     
-    if( $devdetails->param('suppress-legend') ne 'yes' )
+    if( $devdetails->paramDisabled('suppress-legend') )
     {
         $data->{'param'}{'legend'} = $legend;
     }
@@ -332,12 +341,12 @@ sub discover
     # some parameters need just one-to-one copying
 
     my @hostCopyParams =
-        split('\s*,\s*', $devdetails->param('host-copy-params'));
+        split('\s*,\s*', $devdetails->paramString('host-copy-params'));
     
     foreach my $param ( @copyParams, @hostCopyParams )
     {
         my $val = $devdetails->param( $param );
-        if( length( $val ) > 0 )
+        if( defined($val) )
         {
             $data->{'param'}{$param} = $val;
         }
@@ -367,13 +376,14 @@ sub discover
     $self->{'datadirs'}{$devdetails->param('data-dir')} = 1;
 
     $self->{'session'} = $session;
+    $self->{'maxrepetitions'} = $devdetails->param('snmp-maxrepetitions');
 
     # some discovery modules need to be disabled on per-device basis
 
     my %onlyDevtypes;
     my $useOnlyDevtypes = 0;
     foreach my $devtype ( split('\s*,\s*',
-                                $devdetails->param('only-devtypes') ) )
+                                $devdetails->paramString('only-devtypes') ) )
     {
         $onlyDevtypes{$devtype} = 1;
         $useOnlyDevtypes = 1;
@@ -381,7 +391,7 @@ sub discover
 
     my %disabledDevtypes;
     foreach my $devtype ( split('\s*,\s*',
-                                $devdetails->param('disable-devtypes') ) )
+                                $devdetails->paramString('disable-devtypes') ) )
     {
         $disabledDevtypes{$devtype} = 1;
     }
@@ -428,39 +438,36 @@ sub discover
     }
     push( @{$self->{'devdetails'}{$subtree}}, $devdetails );
 
-    my $define_tokensets = $devdetails->param('define-tokensets');
-    if( defined( $define_tokensets ) and length( $define_tokensets ) > 0 )
+    foreach my $pair
+        ( split(/\s*;\s*/, $devdetails->paramString('define-tokensets') ) )
     {
-        foreach my $pair ( split(/\s*;\s*/, $define_tokensets ) )
+        my( $tset, $description ) = split( /\s*:\s*/, $pair );
+        my $params = {};
+        
+        if( $tset !~ /^[a-z][a-z0-9-_]*$/ )
         {
-            my( $tset, $description ) = split( /\s*:\s*/, $pair );
-            my $params = {};
-            
-            if( $tset !~ /^[a-z][a-z0-9-_]*$/ )
-            {
-                Error('Invalid name for tokenset: ' . $tset);
-                $ok = 0;
-            }
-            elsif( length( $description ) == 0 )
-            {
-                Error('Missing description for tokenset: ' . $tset);
-                $ok = 0;
-            }
-            else
-            {
-                $params->{'comment'} = $description;
-            }
-
-            my $v = $devdetails->param($tset . '-tokenset-rrgraph-view');
-            if( defined($v) and length($v) > 0 )
-            {
-                $params->{'rrgraph-view'} = $v;
-            }
-
-            if( $ok )
-            {
-                $self->{'define-tokensets'}{$tset} = $params;
-            }
+            Error('Invalid name for tokenset: ' . $tset);
+            $ok = 0;
+        }
+        elsif( not defined($description) or $description eq '' )
+        {
+            Error('Missing description for tokenset: ' . $tset);
+            $ok = 0;
+        }
+        else
+        {
+            $params->{'comment'} = $description;
+        }
+        
+        my $v = $devdetails->param($tset . '-tokenset-rrgraph-view');
+        if( defined($v) )
+        {
+            $params->{'rrgraph-view'} = $v;
+        }
+        
+        if( $ok )
+        {
+            $self->{'define-tokensets'}{$tset} = $params;
         }
     }
 
@@ -524,7 +531,7 @@ sub buildConfig
             # set by previous host
             $cb->setRegistryOverlays( @registryOverlays );            
             
-            if( $devdetails->param('disable-snmpcollector' ) eq 'yes' )
+            if( $devdetails->paramEnabled('disable-snmpcollector' ) )
             {
                 push( @{$data->{'templates'}}, '::viewonly-defaults' );
             }
@@ -533,28 +540,28 @@ sub buildConfig
                 push( @{$data->{'templates'}}, '::snmp-defaults' );
             }
 
-            if( $devdetails->param('rrd-hwpredict' ) eq 'yes' )
+            if( $devdetails->paramEnabled('rrd-hwpredict' ) )
             {
                 push( @{$data->{'templates'}}, '::holt-winters-defaults' );
             }
             
-            if( $devdetails->param('disable-reachability-stats') ne 'yes'
+            if( $devdetails->paramDisabled('disable-reachability-stats') 
                 and
                 (
                  (not defined($devdetails->param('only-devtypes')))
                  or
-                 $devdetails->param('enable-reachability-stats') eq 'yes'
+                 $devdetails->paramEnabled('enable-reachability-stats') 
                 ) )
             {
                 push( @{$data->{'templates'}}, '::snmp-reachability' );
             }
 
             
-            my $devNodeName = $devdetails->param('symbolic-name');
-            if( length( $devNodeName ) == 0 )
+            my $devNodeName = $devdetails->paramString('symbolic-name');
+            if( $devNodeName eq '' )
             {
-                $devNodeName = $devdetails->param('system-id');
-                if( length( $devNodeName ) == 0 )
+                $devNodeName = $devdetails->paramString('system-id');
+                if( $devNodeName eq '' )
                 {
                     $devNodeName = $devdetails->param('snmp-host');
                 }
@@ -564,22 +571,18 @@ sub buildConfig
                                            $data->{'param'},
                                            $data->{'templates'} );
 
-            my $aliases = $devdetails->param('host-aliases');
-            if( length( $aliases ) > 0 )
+            foreach my $alias
+                ( split( '\s*,\s*',
+                         $devdetails->paramString('host-aliases') ) )
             {
-                foreach my $alias ( split( '\s*,\s*', $aliases ) )
-                {
-                    $cb->addAlias( $devNode, $alias );
-                }
+                $cb->addAlias( $devNode, $alias );
             }
 
-            my $includeFiles = $devdetails->param('include-files');
-            if( length( $includeFiles ) > 0 )
+            foreach my $file
+                ( split( '\s*,\s*',
+                         $devdetails->paramString('include-files') ) )
             {
-                foreach my $file ( split( '\s*,\s*', $includeFiles ) )
-                {
-                    $cb->addFileInclusion( $file );
-                }
+                $cb->addFileInclusion( $file );
             }
                     
 
@@ -617,6 +620,7 @@ sub buildConfig
                               $self->{'define-tokensets'}{$tset} );
         }
     }
+    return;
 }
 
 
@@ -709,7 +713,7 @@ sub checkSnmpTable
         # check if the returned oid shares the base of the query
         my $firstOid = (keys %{$result})[0];
         if( Net::SNMP::oid_base_match( $oid, $firstOid ) and
-            length( $result->{$firstOid} ) > 0 )
+            $result->{$firstOid} ne '' )
         {
             return 1;
         }
@@ -718,6 +722,40 @@ sub checkSnmpTable
 }
 
 
+##
+# walk through the table, and return a hashref with index->value content
+
+sub walkSnmpTable
+{
+    my $self = shift;
+    my $oidname = shift;
+
+    my $session = $self->session();
+    my $base = $self->oiddef( $oidname );
+    
+    my $table = $session->get_table
+        ( -baseoid => $base,
+          -maxrepetitions => $self->{'maxrepetitions'} );
+    
+    if( not defined($table) )
+    {
+        return {};
+    }
+    
+    my $prefixLen = length( $base ) + 1;
+    my $ret = {};
+    
+    while( my( $oid, $value ) = each %{$table} )
+    {
+        my $INDEX = substr( $oid, $prefixLen );
+        $ret->{$INDEX} = $value;
+    }
+
+    return $ret;
+}
+
+
+
 ##
 # Check if given OID is present
 
@@ -733,7 +771,7 @@ sub checkSnmpOID
     if( $session->error_status() == 0 and
         defined($result) and
         defined($result->{$oid}) and
-        length($result->{$oid}) > 0 )
+        $result->{$oid} ne '' )
     {
         return 1;
     }
@@ -778,6 +816,11 @@ sub oidBaseMatch
     my $base_oid = shift;
     my $oid = shift;
 
+    if( not defined($oid) )
+    {
+        return undef;
+    }
+    
     if( $base_oid =~ /^\D/ )
     {
         $base_oid = $self->oiddef( $base_oid );
@@ -785,337 +828,10 @@ sub oidBaseMatch
     return Net::SNMP::oid_base_match( $base_oid, $oid );
 }
 
-##
-# some discovery modules need to adjust max-msg-size
-
-sub setMaxMsgSize
-{
-    my $self = shift;
-    my $devdetails = shift;
-    my $msgsize = shift;
-    my $opt = shift;
-
-    $opt = {} unless defined($opt);
-
-    if( (not $opt->{'only_v1_and_v2'}) or $self->session()->version() != 3 )
-    {
-        $self->session()->max_msg_size($msgsize);
-        $devdetails->data()->{'param'}{'snmp-max-msg-size'} = $msgsize;
-    }
-}
 
     
 
 
-###########################################################################
-####  Torrus::DevDiscover::DevDetails: the information container for a device
-####
-
-package Torrus::DevDiscover::DevDetails;
-
-use strict;
-use Torrus::RPN;
-use Torrus::Log;
-
-sub new
-{
-    my $self = {};
-    my $class = shift;
-    bless $self, $class;
-
-    $self->{'params'}   = {};
-    $self->{'snmpvars'} = {}; # SNMP results stored here
-    $self->{'devtype'}  = {}; # Device types
-    $self->{'caps'}     = {}; # Device capabilities
-    $self->{'data'}     = {}; # Discovery data
-
-    return $self;
-}
-
-
-sub setParams
-{
-    my $self = shift;
-    my $params = shift;
-
-    while( my ($param, $value) = each %{$params} )
-    {
-        $self->{'params'}->{$param} = $value;
-    }
-}
-
-
-sub setParam
-{
-    my $self = shift;
-    my $param = shift;
-    my $value = shift;
-
-    $self->{'params'}->{$param} = $value;
-}
-
-
-sub param
-{
-    my $self = shift;
-    my $name = shift;
-    return $self->{'params'}->{$name};
-}
-
-
-##
-# store the query results for later use
-
-sub storeSnmpVars
-{
-    my $self = shift;
-    my $vars = shift;
-
-    while( my( $oid, $value ) = each %{$vars} )
-    {
-        if( $oid !~ /^\d[0-9.]+\d$/o )
-        {
-            Error("Invalid OID syntax: '$oid'");
-        }
-        else
-        {
-            $self->{'snmpvars'}{$oid} = $value;
-            
-            while( length( $oid ) > 0 )
-            {
-                $oid =~ s/\d+$//o;
-                $oid =~ s/\.$//o;
-                if( not exists( $self->{'snmpvars'}{$oid} ) )
-                {
-                    $self->{'snmpvars'}{$oid} = undef;
-                }
-            }
-        }
-    }
-
-    # Clean the cache of sorted OIDs
-    $self->{'sortedoids'} = undef;
-}
-
-##
-# check if the stored query results have such OID prefix
-
-sub hasOID
-{
-    my $self = shift;
-    my $oid = shift;
-
-    my $found = 0;
-    if( exists( $self->{'snmpvars'}{$oid} ) )
-    {
-        $found = 1;
-    }
-    return $found;
-}
-
-##
-# get the value of stored SNMP variable
-
-sub snmpVar
-{
-    my $self = shift;
-    my $oid = shift;
-    return $self->{'snmpvars'}{$oid};
-}
-
-##
-# get the list of table indices for the specified prefix
-
-sub getSnmpIndices
-{
-    my $self = shift;
-    my $prefix = shift;
-
-    # Remember the sorted OIDs, as sorting is quite expensive for large
-    # arrays.
-    
-    if( not defined( $self->{'sortedoids'} ) )
-    {
-        $self->{'sortedoids'} = [];
-        push( @{$self->{'sortedoids'}},
-              Net::SNMP::oid_lex_sort( keys %{$self->{'snmpvars'}} ) );
-    }
-        
-    my @ret;
-    my $prefixLen = length( $prefix ) + 1;
-    my $matched = 0;
-
-    foreach my $oid ( @{$self->{'sortedoids'}} )
-    {
-        if( defined($self->{'snmpvars'}{$oid} ) )
-        {
-            if( Net::SNMP::oid_base_match( $prefix, $oid ) )
-            {
-                # Extract the index from OID
-                my $index = substr( $oid, $prefixLen );
-                push( @ret, $index );
-                $matched = 1;
-            }
-            elsif( $matched )
-            {
-                last;
-            }
-        }
-    }
-    return @ret;
-}
-
-
-##
-# device type is the registered discovery module name
-
-sub setDevType
-{
-    my $self = shift;
-    my $type = shift;
-    $self->{'devtype'}{$type} = 1;
-}
-
-sub isDevType
-{
-    my $self = shift;
-    my $type = shift;
-    return $self->{'devtype'}{$type};
-}
-
-sub getDevTypes
-{
-    my $self = shift;
-    return keys %{$self->{'devtype'}};
-}
-
-##
-# device capabilities. Each discovery module may define its own set of
-# capabilities and use them for information exchange between checkdevtype(),
-# discover(), and buildConfig() of its own and dependant modules
-
-sub setCap
-{
-    my $self = shift;
-    my $cap = shift;
-    Debug('Device capability: ' . $cap);
-    $self->{'caps'}{$cap} = 1;
-}
-
-sub hasCap
-{
-    my $self = shift;
-    my $cap = shift;
-    return $self->{'caps'}{$cap};
-}
-
-sub clearCap
-{
-    my $self = shift;
-    my $cap = shift;
-    Debug('Clearing device capability: ' . $cap);
-    if( exists( $self->{'caps'}{$cap} ) )
-    {
-        delete $self->{'caps'}{$cap};
-    }
-}
-
-
-
-sub data
-{
-    my $self = shift;
-    return $self->{'data'};
-}
-
-
-sub screenSpecialChars
-{
-    my $self = shift;
-    my $txt = shift;
-
-    $txt =~ s/:/{COLON}/gm;
-    $txt =~ s/;/{SEMICOL}/gm;
-    $txt =~ s/%/{PERCENT}/gm;
-
-    return $txt;
-}
-
-
-sub applySelectors
-{
-    my $self = shift;
-
-    my $selList = $self->param('selectors');
-    return if not defined( $selList );
-
-    my $reg = \%Torrus::DevDiscover::selectorsRegistry;
-    
-    foreach my $sel ( split('\s*,\s*', $selList) )
-    {
-        my $type = $self->param( $sel . '-selector-type' );
-        if( not defined( $type ) )
-        {
-            Error('Parameter ' . $sel . '-selector-type must be defined ' .
-                  'for ' . $self->param('snmp-host'));
-        }
-        elsif( not exists( $reg->{$type} ) )
-        {
-            Error('Unknown selector type: ' . $type .
-                  ' for ' . $self->param('snmp-host'));
-        }
-        else
-        {
-            Debug('Initializing selector: ' . $sel);
-            
-            my $treg = $reg->{$type};
-            my @objects = &{$treg->{'getObjects'}}( $self, $type );
-
-            foreach my $object ( @objects )
-            {
-                Debug('Checking object: ' .
-                      &{$treg->{'getObjectName'}}( $self, $object, $type ));
-
-                my $expr = $self->param( $sel . '-selector-expr' );
-                $expr = '1' if length( $expr ) == 0;
-
-                my $callback = sub
-                {
-                    my $attr = shift;
-                    my $checkval = $self->param( $sel . '-' . $attr );
-                    
-                    Debug('Checking attribute: ' . $attr .
-                          ' and value: ' . $checkval);
-                    my $ret = &{$treg->{'checkAttribute'}}( $self,
-                                                            $object, $type,
-                                                            $attr, $checkval );
-                    Debug(sprintf('Returned value: %d', $ret));
-                    return $ret;                    
-                };
-                
-                my $rpn = new Torrus::RPN;
-                my $result = $rpn->run( $expr, $callback );
-                Debug('Selector result: ' . $result);
-                if( $result )
-                {
-                    my $actions = $self->param( $sel . '-selector-actions' );
-                    foreach my $action ( split('\s*,\s*', $actions) )
-                    {
-                        my $arg =
-                            $self->param( $sel . '-' . $action . '-arg' );
-                        $arg = 1 if not defined( $arg );
-                        
-                        Debug('Applying action: ' . $action .
-                              ' with argument: ' . $arg);
-                        &{$treg->{'applyAction'}}( $self, $object, $type,
-                                                   $action, $arg );
-                    }
-                }
-            }
-        }
-    }
-}    
-
 1;
 
 
