@@ -14,7 +14,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # AxxessIT Ethernet over SDH switches, also known as
@@ -34,6 +33,8 @@
 package Torrus::DevDiscover::AxxessIT;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -94,6 +95,11 @@ sub checkdevtype
     my $devdetails = shift;
 
     my $sysObjID = $devdetails->snmpVar( $dd->oiddef('sysObjectID') );
+    if( not defined($sysObjID) )
+    {
+        return 0;
+    }
+    
     if( index( $sysObjID, $dd->oiddef('axxEdgeTypes') ) == 0 )
     {
         $devdetails->setCap('axxEdge');
@@ -209,7 +215,8 @@ sub discover
 
                 $interface->{'axxInterfaceComment'} =
                     sprintf( $map{$type}{'ifComment'}, $slot, $port );
-                if( length( $desc ) > 0 )
+                
+                if( defined($desc) and $desc ne '' )
                 {
                     $interface->{'axxInterfaceComment'} .= ' (' . $desc . ')';
                 }
@@ -283,7 +290,8 @@ sub discover
                 sprintf( '%s %d', $type, $port );
             
             $interface->{'axxInterfaceComment'} = '';
-            if( length( $portName ) > 0 )
+            
+            if( defined($portName) and $portName ne '' )
             {
                 $interface->{'axxInterfaceComment'} = $portName;
             }
@@ -338,6 +346,7 @@ sub buildConfig
     my $cb = shift;
     my $devNode = shift;
 
+    return;
 }
 
 
