@@ -15,7 +15,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-# $Id$
 # Marc Haber <mh+torrus-devel@zugschlus.de>
 # Redesigned by Stanislav Sinyagin
 
@@ -24,6 +23,8 @@
 package Torrus::DevDiscover::BetterNetworks;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -186,7 +187,7 @@ sub discover
             $data->{'BNEsensor'}{$INDEX}{'leafName'} = $leafName;            
         }
 
-        if( scalar( %{$data->{'BNEsensor'}} ) > 0 )
+        if( scalar(keys %{$data->{'BNEsensor'}}) > 0 )
         {
             $devdetails->setCap('BNEsensor');
 
@@ -224,6 +225,8 @@ sub buildConfig
                           ['BetterNetworks::betternetworks-sensor'] );
         }
     }
+
+    return;
 }
 
 
