@@ -14,12 +14,16 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Shawn Ferry <sferry at sevenspace dot com> <lalartu at obscure dot org>
 
+# there are lots of errors, but we cannot re-test the module yet.
+## no critic (Variables::ProhibitReusedNames)
+
 package Torrus::DevDiscover::EmpireSystemedge;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -183,8 +187,6 @@ sub checkdevtype
     my $session = $dd->session();
     my $data = $devdetails->data();
 
-    my $session = $dd->session();
-
     if( not $dd->checkSnmpTable( 'empire' ) )
     {
         return 0;
@@ -675,9 +677,9 @@ sub buildConfig
 
 
     # Performance Subtree
-    my $subtreeName= "System_Performance";
+    $subtreeName= "System_Performance";
 
-    my $param = {
+    $param = {
         'precedence'     => '-900',
         'comment'        => 'System, CPU and memory statistics'
         };
@@ -785,6 +787,8 @@ sub buildConfig
         }
 
     }
+
+    return;
 }
 
 
