@@ -14,7 +14,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Shawn Ferry <sferry at sevenspace dot com> <lalartu at obscure dot org>
 
 # F5 BigIp Load Balancer
@@ -22,6 +21,8 @@
 package Torrus::DevDiscover::F5BigIp;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -140,7 +141,7 @@ sub discover
     if( not defined( $data->{'param'}{'snmp-oids-per-pdu'} ) )
     {
         my $oidsPerPDU = $devdetails->param('F5BigIp::snmp-oids-per-pdu');
-        if( $oidsPerPDU == 0 )
+        if( not defined($oidsPerPDU) or $oidsPerPDU == 0 )
         {
             $oidsPerPDU = 10;
         }
@@ -162,7 +163,6 @@ sub discover
         return 1;
     }
 
-    my $product_name;
     my $product_name;
     my $result = $dd->retrieveSnmpOIDs( '4.x_globalAttrProductCode' );
     my $product_code = $result->{'4.x_globalAttrProductCode'};
@@ -497,8 +497,9 @@ sub buildConfig
 
                 $server->{'precedence'} = '-100';
 
-                $cb->addSubtree( $poolMemberPoolTree, $server->{'nick'}, $server,
-                              [ 'F5BigIp::BigIp_4.x_poolMember' ] );
+                $cb->addSubtree( $poolMemberPoolTree, $server->{'nick'},
+                                 $server,
+                                 [ 'F5BigIp::BigIp_4.x_poolMember' ] );
             }
         }
     }
@@ -530,6 +531,7 @@ sub buildConfig
 
     }
 
+    return;
 }
 
 
