@@ -14,7 +14,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 # Shawn Ferry <sferry at sevenspace dot com> <lalartu at obscure dot org>
 
@@ -25,6 +24,8 @@
 package Torrus::DevDiscover::MicrosoftWindows;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -187,6 +188,8 @@ sub buildConfig
         my $iisNode = $cb->addSubtree( $devNode, 'MS_IIS', $iisParam,
                                        \@iisTemplates );
     }
+
+    return;
 }
 
 
