@@ -14,7 +14,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Shawn Ferry <sferry at sevenspace dot com> <lalartu at obscure dot org>
 
 # RDBMS MIB
@@ -22,6 +21,8 @@
 package Torrus::DevDiscover::RFC1697_RDBMS;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -213,8 +214,7 @@ sub buildConfig
 
     my $appNode = $cb->addSubtree( $devNode, 'Applications', $appParam );
 
-    my $param = { };
-    my $oraNode = $cb->addSubtree( $appNode, 'Oracle', $param );
+    my $oraNode = $cb->addSubtree( $appNode, 'Oracle' );
 
     if( $devdetails->hasCap('RDBMS::DbTable') )
     {
@@ -228,6 +228,8 @@ sub buildConfig
         }
 
     }
+
+    return;
 }
 
 
