@@ -14,18 +14,18 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Renderer::RRDtool;
-
 use strict;
+use warnings;
 
 use Torrus::ConfigTree;
 use Torrus::RPN;
 use Torrus::Log;
 
 use RRDs;
+use IO::File;
 
 # All our methods are imported by Torrus::Renderer;
 
@@ -76,13 +76,16 @@ sub render_rrgraph
           $self->rrd_make_graph_opts( $config_tree, $token, $view ) );
 
     my $dstype = $config_tree->getNodeParam($token, 'ds-type');
-
+        
     if( $dstype eq 'rrd-multigraph' )
     {
         $self->rrd_make_multigraph( $config_tree, $token, $view, $obj );
     }
     else
     {
+        my $showmax = 0;
+        my $max_dname = $obj->{'dname'} . '_Max';
+        
         my $leaftype = $config_tree->getNodeParam($token, 'leaf-type');
 
         # Handle DEFs and CDEFs
@@ -100,6 +103,22 @@ sub render_rrgraph
                 $self->rrd_make_holtwinters( $config_tree, $token,
                                              $view, $obj );
             }
+            else
+            {
+                if( $self->rrd_if_showmax($config_tree, $token, $view) )
+                {
+                    my $step =
+                        $self->rrd_maxline_step( $config_tree, $view );
+
+                    my $maxdef =
+                        $self->rrd_make_def( $config_tree, $token,
+                                             $max_dname, 'MAX',
+                                             {'step' => $step});
+                    
+                    push( @{$obj->{'args'}{'defs'}}, $maxdef );
+                    $showmax = 1;
+                }
+            }
         }
         elsif( $leaftype eq 'rrd-cdef' )
         {
@@ -107,6 +126,20 @@ sub render_rrgraph
             push( @{$obj->{'args'}{'defs'}},
                   $self->rrd_make_cdef($config_tree, $token,
                                        $obj->{'dname'}, $expr) );
+            
+            if( $self->rrd_if_showmax($config_tree, $token, $view) )
+            {
+                my $step =
+                    $self->rrd_maxline_step( $config_tree, $view );
+                
+                push( @{$obj->{'args'}{'defs'}}, 
+                      $self->rrd_make_cdef( $config_tree, $token,
+                                            $max_dname, $expr,
+                                            {'force_function' => 'MAX',
+                                             'step' => $step} ) );
+                
+                $showmax = 1;
+            }
         }
         else
         {
@@ -115,6 +148,12 @@ sub render_rrgraph
         }
 
         $self->rrd_make_graphline( $config_tree, $token, $view, $obj );
+
+        if( $showmax )
+        {
+            $self->rrd_make_maxline( $max_dname, $config_tree,
+                                     $token, $view, $obj );
+        }            
     }
 
     return(undef) if $obj->{'error'};
@@ -135,9 +174,23 @@ sub render_rrgraph
     }
     Debug("RRDs::graph arguments: " . join(' ', @args));
 
-    $self->tz_set();
-    &RRDs::graph( $outfile, @args );
-    $self->tz_restore();
+    # localize the TZ enviromennt for the child process
+    {
+        my $tz = $self->{'options'}->{'variables'}->{'TZ'};
+        if( not defined($tz) )
+        {
+            $tz = $ENV{'TZ'};
+        }
+
+        local $ENV{'TZ'};
+        if( defined($tz) )
+        {
+            $ENV{'TZ'} = $tz;
+        }
+        
+        &RRDs::graph( $outfile, @args );
+    }
+
     my $ERR=RRDs::error;
     if( $ERR )
     {
@@ -228,9 +281,24 @@ sub render_rrprint
     Debug("RRDs::graph arguments: " . join(' ', @args));
 
     my $printout;
-    $self->tz_set();
-    ($printout, undef, undef) = RRDs::graph('/dev/null', @args);
-    $self->tz_restore();
+
+    # localize the TZ enviromennt for the child process
+    {
+        my $tz = $self->{'options'}->{'variables'}->{'TZ'};
+        if( not defined($tz) )
+        {
+            $tz = $ENV{'TZ'};
+        }
+        
+        local $ENV{'TZ'};
+        if( defined($tz) )
+        {
+            $ENV{'TZ'} = $tz;
+        }
+        
+        ($printout, undef, undef) = RRDs::graph('/dev/null', @args);
+    }
+
     my $ERR=RRDs::error;
     if( $ERR )
     {
@@ -239,15 +307,16 @@ sub render_rrprint
         return undef;
     }
 
-    if( not open(OUT, ">$outfile") )
+    my $fh = IO::File->new($outfile, 'w');
+    if( not defined($fh) )
     {
         Error("Cannot open $outfile for writing: $!");
         return undef;
     }
     else
     {
-        printf OUT ("%s\n", join(':', @{$printout}));
-        close OUT;
+        $fh->printf("%s\n", join(':', @{$printout}));
+        $fh->close();
     }
 
     return( $config_tree->getParam($view, 'expires')+time(), 'text/plain' );
@@ -269,6 +338,8 @@ sub rrd_make_multigraph
     # We need this to refer to some existing variable name
     $obj->{'dname'} = $dsNames[0];
 
+    my $showmax = $self->rrd_if_showmax($config_tree, $token, $view);
+        
     # Analyze the drawing order
     my %dsOrder;
     foreach my $dname ( @dsNames )
@@ -316,13 +387,14 @@ sub rrd_make_multigraph
             }
         }
 
-        my $legend;
+        my $legend = '';
+        my $ds_expr;
         
         if( $dograph or $gprint_this )
         {
-            my $expr = $config_tree->getNodeParam($token, 'ds-expr-'.$dname);
+            $ds_expr = $config_tree->getNodeParam($token, 'ds-expr-'.$dname);
             my @cdefs =
-                $self->rrd_make_cdef($config_tree, $token, $dname, $expr);
+                $self->rrd_make_cdef($config_tree, $token, $dname, $ds_expr);
             if( not scalar(@cdefs) )
             {
                 $obj->{'error'} = 1;
@@ -388,14 +460,57 @@ sub rrd_make_multigraph
                 $stack = '';
             }
                 
+            if( $showmax and ($stack eq '') )
+            {
+                my $max_dname = $dname . '_Max';
+                
+                my $p_maxlinestyle =
+                    $config_tree->getNodeParam($token, 'maxline-style-'.$dname);
+                
+                my $p_maxlinecolor =
+                    $config_tree->getNodeParam($token, 'maxline-color-'.$dname);
+
+                my $step =
+                    $self->rrd_maxline_step( $config_tree, $view );
+                    
+                if( defined($p_maxlinestyle) and defined($p_maxlinecolor) )
+                {
+                    my @cdefs =
+                        $self->rrd_make_cdef($config_tree, $token,
+                                             $max_dname, $ds_expr,
+                                             {'force_function' => 'MAX',
+                                              'step' => $step});
+                    if( not scalar(@cdefs) )
+                    {
+                        $obj->{'error'} = 1;
+                        next;
+                    }
+            
+                    push( @{$obj->{'args'}{'defs'}}, @cdefs );
+
+                    my $max_linestyle = $self->mkline( $p_maxlinestyle );
+                    my $max_linecolor = $self->mkcolor( $p_maxlinecolor );
+                    if( defined( $alpha ) )
+                    {
+                        $max_linecolor .= $alpha;
+                    }
+                    
+                    push( @{$obj->{'args'}{'line'}},
+                          sprintf( '%s:%s%s',
+                                   $max_linestyle,
+                                   $max_dname,
+                                   $max_linecolor ) );
+                }
+            }
+
             push( @{$obj->{'args'}{'line'}},
                   sprintf( '%s:%s%s%s%s', $linestyle, $dname,
                            $linecolor,
-                           length($legend) > 0 ? ':'.$legend.'\l' : '',
+                           ($legend ne '') ? ':'.$legend.'\l' : '',
                            $stack ) );
-            
         }
     }
+    return;
 }
 
 
@@ -489,8 +604,11 @@ sub rrd_make_holtwinters
           sprintf( 'TICK:%sfail%s:1.0:%s',
                    $dname, $hw_fail_color,
                    $Torrus::Renderer::hwGraphLegend ? 'Failures':'') );
+    return;
 }
 
+
+
 sub rrd_make_graphline
 {
     my $self = shift;
@@ -517,7 +635,7 @@ sub rrd_make_graphline
     }
     
     my $styleval = $config_tree->getNodeParam($token, 'line-style');
-    if( not defined( $styleval ) or length( $styleval ) == 0 )
+    if( not defined($styleval)  )
     {
         $styleval = $config_tree->getParam($view, 'line-style');
     }
@@ -525,7 +643,7 @@ sub rrd_make_graphline
     my $linestyle = $self->mkline( $styleval );
 
     my $colorval = $config_tree->getNodeParam($token, 'line-color');
-    if( not defined( $colorval ) or length( $colorval ) == 0 )
+    if( not defined($colorval) )
     {
         $colorval = $config_tree->getParam($view, 'line-color');
     }
@@ -542,7 +660,67 @@ sub rrd_make_graphline
 
     push( @{$obj->{'args'}{'line'}},
           sprintf( '%s:%s%s%s', $linestyle, $obj->{'dname'}, $linecolor,
-                   length($legend) > 0 ? ':'.$legend.'\l' : '' ) );
+                   ($legend ne '') ? ':'.$legend.'\l' : '' ) );
+    return;
+}
+
+
+sub rrd_make_maxline
+{
+    my $self = shift;
+    my $max_dname = shift;
+    my $config_tree = shift;
+    my $token = shift;
+    my $view = shift;
+    my $obj = shift;
+
+    my $legend;
+    
+    my $disable_legend = $config_tree->getParam($view, 'disable-legend');
+    if( not defined($disable_legend) or $disable_legend ne 'yes' )
+    {
+        $legend = $config_tree->getNodeParam($token, 'graph-legend');
+        if( defined( $legend ) )
+        {
+            $legend =~ s/:/\\:/g;
+        }
+    }
+
+    if( not defined( $legend ) )
+    {
+        $legend = '';
+    }
+    else
+    {
+        $legend = 'Max ' . $legend;
+    }
+    
+    my $styleval = $config_tree->getNodeParam($token, 'maxline-style');
+    if( not defined($styleval)  )
+    {
+        $styleval = $config_tree->getParam($view, 'maxline-style');
+    }
+    
+    my $linestyle = $self->mkline( $styleval );
+
+    my $colorval = $config_tree->getNodeParam($token, 'maxline-color');
+    if( not defined($colorval) )
+    {
+        $colorval = $config_tree->getParam($view, 'maxline-color');
+    }
+    
+    my $linecolor = $self->mkcolor( $colorval );
+
+    if( $self->rrd_if_gprint( $config_tree, $token ) )
+    {
+        $self->rrd_make_gprint( $max_dname, $legend,
+                                $config_tree, $token, $view, $obj );
+    }
+
+    push( @{$obj->{'args'}{'line'}},
+          sprintf( '%s:%s%s%s', $linestyle, $max_dname, $linecolor,
+                   ($legend ne '') ? ':'.$legend.'\l' : '' ) );
+    return;
 }
 
 
@@ -585,6 +763,7 @@ sub rrd_make_hrules
             }
         }
     }
+    return;
 }
 
 
@@ -641,6 +820,7 @@ sub rrd_make_decorations
             push( @{$obj->{'args'}{$array}}, $decor->{$order}{'line'} );
         }
     }
+    return;
 }
 
 # Takes the parameters from the view, and composes the list of
@@ -735,7 +915,7 @@ sub rrd_make_graph_opts
     if( not defined( $disable_title ) or $disable_title ne 'yes' )
     {
         my $title = $config_tree->getNodeParam($token, 'graph-title');
-        if( not defined( $title ) or length( $title ) == 0 )
+        if( not defined($title) )
         {
             $title = ' ';
         }
@@ -748,7 +928,7 @@ sub rrd_make_graph_opts
     {
         my $vertical_label =
             $config_tree->getNodeParam($token, 'vertical-label');
-        if( defined( $vertical_label ) and length( $vertical_label ) > 0 )
+        if( defined( $vertical_label ) )
         {
             push( @args, '--vertical-label', $vertical_label );
         }
@@ -762,7 +942,7 @@ sub rrd_make_graph_opts
         {
             my $limit =
                 $config_tree->getNodeParam($token, 'graph-lower-limit');
-            if( defined($limit) and length( $limit ) > 0 )
+            if( defined($limit) )
             {
                 push( @args, '--lower-limit', $limit );
             }
@@ -773,7 +953,7 @@ sub rrd_make_graph_opts
         {
             my $limit =
                 $config_tree->getNodeParam($token, 'graph-upper-limit');
-            if( defined($limit) and length( $limit ) > 0 )
+            if( defined($limit) )
             {
                 push( @args, '--upper-limit', $limit );
             }
@@ -803,6 +983,7 @@ sub rrd_make_def
     my $token = shift;
     my $dname = shift;
     my $cf = shift;
+    my $opts = shift;
 
     my $datafile = $config_tree->getNodeParam($token, 'data-file');
     my $dataddir = $config_tree->getNodeParam($token, 'data-dir');
@@ -819,8 +1000,15 @@ sub rrd_make_def
     {
         $cf = $config_tree->getNodeParam($token, 'rrd-cf');
     }
-    return sprintf( 'DEF:%s=%s:%s:%s',
-                    $dname, $rrdfile, $ds, $cf );
+
+    my $def_options = '';
+    if( defined($opts) and defined($opts->{'step'}) )
+    {
+        $def_options .= ':step=' . $opts->{'step'};
+    }
+    
+    return sprintf( 'DEF:%s=%s:%s:%s%s',
+                    $dname, $rrdfile, $ds, $cf, $def_options );
 }
 
 
@@ -839,6 +1027,7 @@ sub rrd_make_cdef
     my $token = shift;
     my $dname = shift;
     my $expr  = shift;
+    my $opts = shift;
 
     my @args = ();
     my $ok = 1;
@@ -854,7 +1043,11 @@ sub rrd_make_cdef
         my ($noderef, $timeoffset) = @_;
 
         my $function;
-        if( $noderef =~ s/^(.+)\@// )
+        if( defined($opts) and defined($opts->{'force_function'}) )
+        {
+            $function = $opts->{'force_function'};
+        }
+        elsif( $noderef =~ s/^(.+)\@// )
         {
             $function = $1;
         }
@@ -865,7 +1058,7 @@ sub rrd_make_cdef
             $cf = $function;
         }
         
-        my $leaf = length($noderef) > 0 ?
+        my $leaf = ($noderef ne '') ?
             $config_tree->getRelative($token, $noderef) : $token;
 
         my $varname = $dname . sprintf('%.2d', $ds_couter++);
@@ -877,6 +1070,10 @@ sub rrd_make_cdef
         }
         else
         {
+            if( defined($opts) and defined($opts->{'step'}) )
+            {
+                $defstring .= ':step=' . $opts->{'step'};
+            }
             push( @args, $defstring );
         }
         return $varname;
@@ -904,6 +1101,61 @@ sub rrd_if_gprint
     return 1;
 }
 
+
+# determine if MAX line should be drawn
+sub rrd_if_showmax
+{
+    my $self = shift;
+    my $config_tree = shift;
+    my $token = shift;
+    my $view = shift;
+
+    my $disable = $config_tree->getNodeParam($token, 'graph-disable-maxline');
+    if( defined( $disable ) and $disable eq 'yes' )
+    {
+        return 0;
+    }
+
+    if( $self->{'options'}->{'variables'}->{'Gmaxline'} )
+    {
+        return 1;
+    }
+
+    my $enable = $config_tree->getParam($view, 'draw-maxline');
+    if( defined($enable) and $enable eq 'yes' )
+    {
+        return 1;
+    }
+
+    return 0;
+}
+
+# determine the aggregation step for MAX line
+sub rrd_maxline_step
+{
+    my $self = shift;
+    my $config_tree = shift;
+    my $view = shift;
+
+    my $step = $config_tree->getParam($view, 'maxline-step');    
+    if( not defined($step) )
+    {
+        $step = 86400;
+    }
+
+    my $var = $self->{'options'}->{'variables'}->{'Gmaxlinestep'};
+
+    if( defined($var) )
+    {
+        $step = $var;
+    }
+
+    return $step;
+}
+    
+
+    
+
 sub rrd_make_gprint
 {
     my $self = shift;
@@ -917,7 +1169,7 @@ sub rrd_make_gprint
     my @args = ();
 
     my $gprintValues = $config_tree->getParam($view, 'gprint-values');
-    if( defined( $gprintValues ) and length( $gprintValues ) > 0 )
+    if( defined( $gprintValues ) )
     {
         foreach my $gprintVal ( split(',', $gprintValues ) )
         {
@@ -928,6 +1180,7 @@ sub rrd_make_gprint
     }
 
     push( @{$obj->{'args'}{'line'}}, @args );
+    return;
 }
             
 
@@ -940,15 +1193,16 @@ sub rrd_make_gprint_header
     my $obj = shift;
 
     my $gprintValues = $config_tree->getParam($view, 'gprint-values');
-    if( defined( $gprintValues ) and length( $gprintValues ) > 0 )
+    if( defined( $gprintValues ) )
     {
         my $gprintHeader = $config_tree->getParam($view, 'gprint-header');
-        if( defined( $gprintHeader ) and length( $gprintHeader ) > 0 )
+        if( defined( $gprintHeader ) )
         {
             push( @{$obj->{'args'}{'line'}},
                   'COMMENT:' . $gprintHeader . '\l' );
         }
     }
+    return;
 }
        
 
@@ -999,51 +1253,6 @@ sub mkline
 }
 
 
-sub tz_set
-{
-    my $self = shift;
-
-    if( defined $ENV{'TZ'} )
-    {
-        Debug("Previous TZ value: " . $ENV{'TZ'});
-        $self->{'tz_defined'} = 1;
-    }
-    else
-    {
-        $self->{'tz_defined'} = 0;
-    }
-
-    if( defined( my $newTZ = $self->{'options'}->{'variables'}->{'TZ'} ) )
-    {
-        Debug("Setting TZ to " . $newTZ);
-        $self->{'tz_old'} = $ENV{'TZ'};
-        $ENV{'TZ'} = $newTZ;
-        $self->{'tz_changed'} = 1;
-    }
-    else
-    {
-        $self->{'tz_changed'} = 0;
-    }
-}
-
-sub tz_restore
-{
-    my $self = shift;
-
-    if( $self->{'tz_changed'} )
-    {
-        if( $self->{'tz_defined'} )
-        {
-            Debug("Restoring TZ back to " . $self->{'tz_old'});
-            $ENV{'TZ'} = $self->{'tz_old'};
-        }
-        else
-        {
-            Debug("Restoring TZ back to undefined");
-            delete $ENV{'TZ'};
-        }
-    }
-}
 
 
 1;
